# eslint-plugin-no-hardcoding
To prevent hardcoding content and to enforce usage of appropriate i18n keys

# Installation
You'll first need to install **ESLint**:

```
$ npm i eslint --save-dev 
```

Next, install ` eslint-plugin-no-hardcoding `:

```
$ npm install eslint-plugin-no-hardcoding --save-dev
```
**Note:** If you installed ESLint globally (using the -g flag) then you must also install `eslint-plugin-no-hardcoding` globally.

# Usage
Add `@zohodesk/no-hardcoding` to the plugins section of your `.eslintrc` configuration file.

```
{
  "plugins": ["@zohodesk/no-hardcoding"]
}
```
Then configure the rules you want to use under the rules section.

```
{
  "rules": {
    "@zohodesk/no-hardcoding/no-hardcoding": [errror,{props:[],objectKeys:[],methods:[],allowedTags:[]}],
    "@zohodesk/no-hardcoding/use-getI18NValue-method-appropriately": [errror,{i18nKeys:[],comments:[]}]
  }
}
```
**Note:** When configuring the severity of the no-hardcoding rule `@zohodesk/no-hardcoding/no-hardcoding` as a second argument in the array, an object with the properties 'methods','props','objectKeys','allowedTags','keys','commentsToIgnoreHardcodingCheck','stringsAllowedAsBlockCommentStart','stringsAllowedAsBlockCommentEnd','dateFormatPatterns' must be mandatorily given. The value of these properties should be an array and entries into the array can be added if required

**Note:** When configuring the severity of the no-hardcoding rule `@zohodesk/no-hardcoding/use-getI18NValue-method-appropriately` as a second argument in the array an object with the properties 'i18nKeys','commentsToIgnoreCheck' must be mandatorily given. The value of these properties should be an array and entries into the array can be added if required

# List of supported rules
- @zohodesk/no-hardcoding/no-hardcoding,
- @zohodesk/no-hardcoding/use-getI18NValue-method-appropriately

# Cases where the @zohodesk/no-hardcoding/no-hardcoding rule will report error:
```
<Container displayLabel='hardcoded string' /> 
```

```
let obj = {
        key: 'hardcoded string'
    }
```

```
let test = "test variable"
```

```
let i18nText = getI18NValue('Incorrect key')
```
When an incorrect key is used inside the i18n method getI18NValue error will be reported

# Cases where the @zohodesk/no-hardcoding/no-hardcoding rule will not report error
1. Hardcoding string in certain props like className, dataId etc are by default not reported as errors. Such props where error should not be thrown can be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
<Container className="test string" dataId='any string' />
```

2. Hardcoding string in certain object properties/keys like className, dataId,iconSize,boxSize etc are by default not reported as errors. Such object keys/properties where error should not be thrown can be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
let obj = { dataId: 'any string'}
```

3. Hardcoding string in certain methods like findIndex,includes, addEventListener etc are by default not reported as errors. Such methods where error should not be thrown can be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
let index = findIndex('hardcoded string')
```

4. Hardcoding strings in certain tags like svg are by default excluded from being reported as error. Such tags can be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
<svg title='hardcoded string' d='test'> </svg>
```

5. Here the string 'Product Information' is actually an i18n key and passed to appropriate method getI18NValue. Hence 
it won't be reported as error. Such hardcoded strings which are correct i18n keys need to be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
let i18nedValue = getI18NValue('Product Information')
``` 
6. 
```
let seqOfNums = "0 0 12 20"
```

7. Here the word "spaced key" is actully a property name in the object 'obj' and hence won't be reported as error
```
let num = obj["spaced key"] 
```

8. Date formats like above won't be reported as hardcoding errors. Such date format patterns can be configured in the configuration object given in the configuration file while configuring the rule(as explained above on how to configure the rules)
```
let dateFormat = 'DD MMM YYYY hh:mm A'
```

9. css properties won't be reported as hardcoding errors
```
let cssProp = '10px #34rfe3' 
```

10. strings given as argument to throw expressions won't be reported as hardcoding errors
``` 
throw 'any string' 
```

11. strings passed as arguments to "Error" won't be reported as hardcoding errors 
``` 
return new Error('any string') 
```

12. strings used for comparison in if conditions won't be reported as hardcoding errors 
```
if(someVar === 'any string'){}
```

13. strings used for comparison in ternary operators won't be reported as hardcoding errors 
``` 
let someVar = "any string" === "another string" ? true : false
```

14. when the comment 'no i18n' is added hardcoding error will not be reported in the particular line
```
let someVar = 'hardcoded string' //no i18n
```

15. if the block comment 'ignore i18n start' and 'ignore i18n end' is added no hardcoding error will be reported in the lines between the comment
``` 
let obj = { //ignore i18n start
    title: 'hardcoded string',
    displayLabel: 'Show tickets',
    text: 'In Progress',
    buttonText: 'Leave now'
} //ignore i18n end
```
